@@ -307,6 +307,8 @@ gdk_pixbuf__jpeg_image_load (FILE *f, GError **error)
 			g_object_unref (pixbuf);
 
 		jpeg_destroy_decompress (&cinfo);
+
+		/* error should have been set by fatal_error_handler () */
 		return NULL;
 	}
 
@@ -527,6 +529,7 @@ gdk_pixbuf__jpeg_image_stop_load (gpointer data, GError **error)
 		g_object_unref (context->pixbuf);
 	
 	/* if we have an error? */
+	context->jerr.error = error;
 	if (sigsetjmp (context->jerr.setjmp_buffer, 1)) {
 		jpeg_destroy_decompress (&context->cinfo);
 	} else {
@@ -595,12 +598,13 @@ gdk_pixbuf__jpeg_image_load_lines (JpegProgContext  *context,
                 context->dptr += nlines * context->pixbuf->rowstride;
 
                 /* send updated signal */
-                (* context->updated_func) (context->pixbuf,
-                                           0,
-                                           cinfo->output_scanline - 1,
-                                           cinfo->image_width,
-                                           nlines,
-                                           context->user_data);
+		if (context->updated_func)
+			(* context->updated_func) (context->pixbuf,
+						   0,
+						   cinfo->output_scanline - 1,
+						   cinfo->image_width,
+						   nlines,
+						   context->user_data);
         }
 
         return TRUE;
@@ -714,8 +718,13 @@ gdk_pixbuf__jpeg_image_load_increment (gpointer data,
 			height = cinfo->image_height;
 			if (context->size_func) {
 				(* context->size_func) (&width, &height, context->user_data);
-				if (width == 0 || height == 0)
+				if (width == 0 || height == 0) {
+					g_set_error (error,
+						     GDK_PIXBUF_ERROR,
+						     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
+						     _("Transformed JPEG has zero width or height."));
 					return FALSE;
+				}
 			}
 			
 			for (cinfo->scale_denom = 2; cinfo->scale_denom <= 8; cinfo->scale_denom *= 2) {
@@ -745,9 +754,10 @@ gdk_pixbuf__jpeg_image_load_increment (gpointer data,
 			context->dptr = context->pixbuf->pixels;
 			
 			/* Notify the client that we are ready to go */
-			(* context->prepared_func) (context->pixbuf,
-                                                    NULL,
-						    context->user_data);
+			if (context->prepared_func)
+				(* context->prepared_func) (context->pixbuf,
+							    NULL,
+							    context->user_data);
 			
 		} else if (!context->did_prescan) {
 			int rc;			
@@ -942,9 +952,7 @@ real_save_jpeg (GdkPixbuf          *pixbuf,
                                        return FALSE;
                                }
                        } else {
-                               g_warning ("Bad option name '%s' passed to JPEG saver",
-                                          *kiter);
-                               return FALSE;
+                               g_warning ("Unrecognized parameter (%s) passed to JPEG saver.", *kiter);
                        }
                
                        ++kiter;
@@ -957,10 +965,7 @@ real_save_jpeg (GdkPixbuf          *pixbuf,
 
        w = gdk_pixbuf_get_width (pixbuf);
        h = gdk_pixbuf_get_height (pixbuf);
-
-       /* no image data? abort */
        pixels = gdk_pixbuf_get_pixels (pixbuf);
-       g_return_val_if_fail (pixels != NULL, FALSE);
 
        /* Allocate a small buffer to convert image data,
 	* and a larger buffer if doing to_callback save.
