@@ -1,3 +1,29 @@
+/* GDK - The GIMP Drawing Kit
+ * Copyright (C) 2000 Elliot Lee
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ */
+
+/*
+ * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
+ * file for a list of people on the GTK+ Team.  See the ChangeLog
+ * files for a list of changes.  These files are distributed with
+ * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
+ */
+
 #include <config.h>
 #include "gdkprivate-fb.h"
 #include "mi.h"
@@ -126,12 +152,6 @@ static void         gdk_shadow_fb_draw_text_wc       (GdkDrawable      *drawable
 						      gint              y,
 						      const GdkWChar   *text,
 						      gint              text_length);
-static void         gdk_shadow_fb_draw_glyphs        (GdkDrawable      *drawable,
-						      GdkGC            *gc,
-						      PangoFont        *font,
-						      gint              x,
-						      gint              y,
-						      PangoGlyphString *glyphs);
 static void         gdk_shadow_fb_draw_drawable      (GdkDrawable      *drawable,
 						      GdkGC            *gc,
 						      GdkPixmap        *src,
@@ -198,7 +218,6 @@ gdk_drawable_impl_fb_class_init (GdkDrawableFBClass *klass)
   drawable_class->draw_points = gdk_shadow_fb_draw_points;
   drawable_class->draw_segments = gdk_shadow_fb_draw_segments;
   drawable_class->draw_lines = gdk_shadow_fb_draw_lines;
-  drawable_class->draw_glyphs = gdk_shadow_fb_draw_glyphs;
   drawable_class->draw_image = gdk_shadow_fb_draw_image;
 #else
   drawable_class->draw_rectangle = gdk_fb_draw_rectangle;
@@ -210,7 +229,6 @@ gdk_drawable_impl_fb_class_init (GdkDrawableFBClass *klass)
   drawable_class->draw_points = gdk_fb_draw_points;
   drawable_class->draw_segments = gdk_fb_draw_segments;
   drawable_class->draw_lines = gdk_fb_draw_lines;
-  drawable_class->draw_glyphs = gdk_fb_draw_glyphs;
   drawable_class->draw_image = gdk_fb_draw_image;
 #endif
   
@@ -433,9 +451,11 @@ gdk_fb_clip_region (GdkDrawable *drawable,
 
   if (gc)
     {
-      if (GDK_GC_FBDATA (gc)->clip_region)
+      GdkRegion *gc = _gdk_gc_get_clip_region (gc);
+      
+      if (clip_region)
 	{
-	  tmpreg = gdk_region_copy (GDK_GC_FBDATA (gc)->clip_region);
+	  tmpreg = gdk_region_copy (clip_region);
 	  gdk_region_offset (tmpreg, private->abs_x + GDK_GC_P (gc)->clip_x_origin,
 			     private->abs_y + GDK_GC_P (gc)->clip_y_origin);
 	  gdk_region_intersect (real_clip_region, tmpreg);
@@ -825,6 +845,7 @@ gdk_fb_draw_text(GdkDrawable    *drawable,
 		 gint            text_length)
 {
   GdkFontPrivateFB *private;
+  GdkDrawableFBData *drawable_private;
   guchar *utf8, *utf8_end;
   PangoGlyphString *glyphs = pango_glyph_string_new ();
   PangoEngineShape *shaper, *last_shaper;
@@ -837,6 +858,7 @@ gdk_fb_draw_text(GdkDrawable    *drawable,
   g_return_if_fail (text != NULL);
 
   private = (GdkFontPrivateFB*) font;
+  drawable_private = GDK_DRAWABLE_FBDATA (drawable);
 
   utf8 = alloca (text_length*2);
 
@@ -878,9 +900,10 @@ gdk_fb_draw_text(GdkDrawable    *drawable,
 	  
 	  pango_shape (start, p - start, &analysis, glyphs);
 
-	  gdk_fb_draw_glyphs (drawable, gc, private->pango_font,
-			      x + PANGO_PIXELS (x_offset), y,
-			      glyphs);
+	  gdk_draw_glyphs (drawable_private->wrapper,
+			   gc, private->pango_font,
+			   x + PANGO_PIXELS (x_offset), y,
+			   glyphs);
 	  
 	  for (i = 0; i < glyphs->num_glyphs; i++)
 	    x_offset += glyphs->glyphs[i].geometry.width;
@@ -900,9 +923,10 @@ gdk_fb_draw_text(GdkDrawable    *drawable,
       
       pango_shape (start, p - start, &analysis, glyphs);
       
-      gdk_fb_draw_glyphs (drawable, gc, private->pango_font,
-			  x + PANGO_PIXELS (x_offset), y,
-			  glyphs);
+      gdk_draw_glyphs (drawable_private->wrapper,
+		       gc, private->pango_font,
+		       x + PANGO_PIXELS (x_offset), y,
+		       glyphs);
     }
   
   pango_glyph_string_free (glyphs);
@@ -1176,93 +1200,6 @@ gdk_fb_drawable_clear (GdkDrawable *d)
   _gdk_windowing_window_clear_area (d, 0, 0, GDK_DRAWABLE_IMPL_FBDATA (d)->width, GDK_DRAWABLE_IMPL_FBDATA (d)->height);
 }
 
-static void
-_gdk_fb_draw_glyphs (GdkDrawable      *drawable,
-		     GdkGC	     *gc,
-		     PangoFont        *font,
-		     gint              x,
-		     gint              y,
-		     PangoGlyphString *glyphs,
-		     GdkRectangle     *bbox)
-{
-  GdkFBDrawingContext fbdc;
-  GdkPixmapFBData pixmap;
-  PangoGlyphInfo *gi;
-  FT_Face face;
-  FT_UInt glyph_index;
-  int i, xpos;
-  int maxy, miny;
-  int topy;
-
-  g_return_if_fail (font);
-
-  gdk_fb_drawing_context_init (&fbdc, drawable, gc, FALSE, TRUE);
-
-  /* Fake its existence as a pixmap */
-
-  ((GTypeInstance *)&pixmap)->g_class = g_type_class_peek (_gdk_pixmap_impl_get_type ());
-  pixmap.drawable_data.abs_x = 0;
-  pixmap.drawable_data.abs_y = 0;
-  pixmap.drawable_data.depth = 78;
-
-  maxy = miny = 0;
-  
-  gi = glyphs->glyphs;
-  for (i = 0, xpos = 0; i < glyphs->num_glyphs; i++, gi++)
-    {
-      if (gi->glyph)
-	{
-	  glyph_index = gi->glyph;
-	  face = pango_ft2_font_get_face (font);
-
-	  if (face)
-	    {
-	      /* Draw glyph */
-	      FT_Load_Glyph (face, glyph_index, FT_LOAD_DEFAULT);
-	      if (face->glyph->format != ft_glyph_format_bitmap)
-		FT_Render_Glyph (face->glyph, ft_render_mode_normal);
-
-	      pixmap.drawable_data.mem = face->glyph->bitmap.buffer;
-	      pixmap.drawable_data.rowstride = face->glyph->bitmap.pitch;
-	      pixmap.drawable_data.width = face->glyph->bitmap.width;
-	      pixmap.drawable_data.height = face->glyph->bitmap.rows;
-
-	      topy = y - face->glyph->bitmap_top + 1;
-	      miny = MIN (miny, topy);
-	      maxy = MAX (maxy, topy + face->glyph->bitmap.rows);
-	      gdk_fb_draw_drawable_3 (drawable, gc, (GdkPixmap *)&pixmap,
-				      &fbdc,
-				      0, 0,
-				      x + PANGO_PIXELS (xpos) + face->glyph->bitmap_left,
-				      topy,
-				      face->glyph->bitmap.width, face->glyph->bitmap.rows);
-	    }
-	}
-      xpos += glyphs->glyphs[i].geometry.width;
-    }
-
-  gdk_fb_drawing_context_finalize (&fbdc);
-
-  if (bbox)
-    {
-      bbox->x = x;
-      bbox->y = miny;
-      bbox->width = xpos;
-      bbox->height = maxy - miny;
-    }
-}
-
-static void
-gdk_fb_draw_glyphs (GdkDrawable      *drawable,
-		    GdkGC	     *gc,
-		    PangoFont        *font,
-		    gint              x,
-		    gint              y,
-		    PangoGlyphString *glyphs)
-{
-  _gdk_fb_draw_glyphs (drawable, gc, font, x, y, glyphs, NULL);
-}
-
 static void
 gdk_fb_draw_image (GdkDrawable *drawable,
 		   GdkGC       *gc,
@@ -1463,25 +1400,6 @@ gdk_shadow_fb_draw_text_wc (GdkDrawable      *drawable,
   gdk_fb_draw_text_wc (drawable, font, gc, x, y, text, text_length);
 }
 
-static void
-gdk_shadow_fb_draw_glyphs (GdkDrawable      *drawable,
-			   GdkGC            *gc,
-			   PangoFont        *font,
-			   gint              x,
-			   gint              y,
-			   PangoGlyphString *glyphs)
-{
-  GdkDrawableFBData *private;
-  GdkRectangle bbox;
-  
-  _gdk_fb_draw_glyphs (drawable, gc, font, x, y, glyphs, &bbox);
-  
-  private = GDK_DRAWABLE_FBDATA (drawable);
-  if (GDK_IS_WINDOW (private->wrapper))
-    gdk_shadow_fb_update (bbox.x + private->abs_x, bbox.y + private->abs_y,
-			  bbox.x + private->abs_x + bbox.width, bbox.y + private->abs_y + bbox.height);
-}
-
 static void
 gdk_shadow_fb_draw_drawable (GdkDrawable      *drawable,
 			     GdkGC            *gc,
@@ -1643,3 +1561,15 @@ gdk_shadow_fb_draw_lines (GdkDrawable      *drawable,
 }
 
 #endif
+
+gboolean
+gdk_draw_rectangle_alpha_libgtk_only (GdkDrawable *drawable,
+				      gint         x,
+				      gint         y,
+				      gint         width,
+				      gint         height,
+				      GdkColor    *color,
+				      guint16      alpha)
+{
+  return FALSE;
+}
