@@ -1335,7 +1335,7 @@ gtk_action_set_short_label (GtkAction	*action,
 
       if (GTK_IS_TOOL_BUTTON (proxy))
 	gtk_tool_button_set_label (GTK_TOOL_BUTTON (proxy), 
-				   action->private_data->label);
+				   action->private_data->short_label);
       else if (GTK_IS_BUTTON (proxy) &&
 	       !gtk_button_get_use_stock (GTK_BUTTON (proxy)))
 	{
@@ -1343,7 +1343,7 @@ gtk_action_set_short_label (GtkAction	*action,
 	  
 	  if (child == NULL || GTK_IS_LABEL (child))
 	    gtk_button_set_label (GTK_BUTTON (proxy), 
-				  action->private_data->label);
+				  action->private_data->short_label);
 	}
     }
 
