@@ -97,6 +97,7 @@ static const LPCWSTR class_descriptors[] = {
 };
 
 static const short element_part_map[] = {
+    BP_CHECKBOX,
     BP_CHECKBOX,
     BP_CHECKBOX,
     BP_PUSHBUTTON,
@@ -358,6 +359,7 @@ xp_theme_get_handle_by_element (XpThemeElement element)
 	    break;
 
 	case XP_THEME_ELEMENT_PRESSED_CHECKBOX:
+	case XP_THEME_ELEMENT_INCONSISTENT_CHECKBOX:
 	case XP_THEME_ELEMENT_CHECKBOX:
 	case XP_THEME_ELEMENT_BUTTON:
 	case XP_THEME_ELEMENT_DEFAULT_BUTTON:
@@ -644,6 +646,23 @@ xp_theme_map_gtk_state (XpThemeElement element, GtkStateType state)
 		}
 	    break;
 
+	case XP_THEME_ELEMENT_INCONSISTENT_CHECKBOX:
+	  switch (state)
+	    {
+	    case GTK_STATE_SELECTED:
+	      ret = CBS_MIXEDPRESSED;
+	      break;
+	    case GTK_STATE_PRELIGHT:
+	      ret = CBS_MIXEDHOT;
+	      break;
+	    case GTK_STATE_INSENSITIVE:
+	      ret = CBS_MIXEDDISABLED;
+	      break;
+	    default:
+	      ret = CBS_MIXEDNORMAL;
+	    }
+	  break;
+
 	case XP_THEME_ELEMENT_PRESSED_CHECKBOX:
 	case XP_THEME_ELEMENT_PRESSED_RADIO_BUTTON:
 	    switch (state)
