@@ -30,6 +30,10 @@
 #include <stdlib.h>
 #include <time.h>
 
+#if CUPS_VERSION_MAJOR > 1 || (CUPS_VERSION_MAJOR == 1 && CUPS_VERSION_MINOR > 1) || (CUPS_VERSION_MAJOR == 1 && CUPS_VERSION_MINOR == 1 && CUPS_VERSION_PATCH >= 20)
+#define HAVE_HTTP_AUTHSTRING 1
+#endif
+
 typedef void (*GtkCupsRequestStateFunc) (GtkCupsRequest *request);
 
 static void _connect            (GtkCupsRequest *request);
@@ -47,6 +51,11 @@ struct _GtkCupsResult
 {
   gchar *error_msg;
   ipp_t *ipp_response;
+  GtkCupsErrorType error_type;
+
+  /* some error types like HTTP_ERROR have a status and a code */
+  int error_status;            
+  int error_code;
 
   guint is_error : 1;
   guint is_ipp_response : 1;
@@ -69,15 +78,20 @@ GtkCupsRequestStateFunc get_states[] = {_connect,
                                         _get_read_data};
 
 static void
-gtk_cups_result_set_error (GtkCupsResult *result, 
-                           const char *error_msg,
+gtk_cups_result_set_error (GtkCupsResult    *result,
+                           GtkCupsErrorType  error_type,
+                           int               error_status,
+                           int               error_code, 
+                           const char       *error_msg,
 			   ...)
 {
   va_list args;
 
   result->is_ipp_response = FALSE;
-
   result->is_error = TRUE;
+  result->error_type = error_type;
+  result->error_status = error_status;
+  result->error_code = error_code;
 
   va_start (args, error_msg);
   result->error_msg = g_strdup_vprintf (error_msg, args);
@@ -202,7 +216,13 @@ gtk_cups_request_read_write (GtkCupsRequest *request)
   if (request->attempts > _GTK_CUPS_MAX_ATTEMPTS && 
       request->state != GTK_CUPS_REQUEST_DONE)
     {
-      gtk_cups_result_set_error (request->result, "Too many failed attempts");
+      /* TODO: should add a status or error code for too many failed attempts */
+      gtk_cups_result_set_error (request->result, 
+                                 GTK_CUPS_ERROR_GENERAL,
+                                 0,
+                                 0, 
+                                 "Too many failed attempts");
+
       request->state = GTK_CUPS_REQUEST_DONE;
       request->poll_state = GTK_CUPS_HTTP_IDLE;
     }
@@ -357,10 +377,15 @@ _find_option_tag (const gchar *option)
     }
 }
 
+/*
+ * Note that this function uses IPP_TAG_JOB, so it is
+ * only suitable for IPP Group 2 attributes.
+ * See RFC 2911.
+ */
 void
 gtk_cups_request_encode_option (GtkCupsRequest *request,
-                                const gchar *option,
-				const gchar *value)
+                                const gchar    *option,
+			        const gchar    *value)
 {
   ipp_tag_t option_tag;
 
@@ -384,7 +409,7 @@ gtk_cups_request_encode_option (GtkCupsRequest *request,
       case IPP_TAG_INTEGER:
       case IPP_TAG_ENUM:
         ippAddInteger (request->ipp_request,
-                       IPP_TAG_OPERATION,
+                       IPP_TAG_JOB,
                        option_tag,
                        option,
                        strtol (value, NULL, 0));
@@ -400,7 +425,7 @@ gtk_cups_request_encode_option (GtkCupsRequest *request,
 	    b = 1;
 	  
           ippAddBoolean(request->ipp_request,
-                        IPP_TAG_OPERATION,
+                        IPP_TAG_JOB,
                         option,
                         b);
         
@@ -432,7 +457,7 @@ gtk_cups_request_encode_option (GtkCupsRequest *request,
 	    upper = lower;
          
           ippAddRange (request->ipp_request,
-                       IPP_TAG_OPERATION,
+                       IPP_TAG_JOB,
                        option,
                        lower,
                        upper);
@@ -460,7 +485,7 @@ gtk_cups_request_encode_option (GtkCupsRequest *request,
             units = IPP_RES_PER_INCH;
           
           ippAddResolution (request->ipp_request,
-                            IPP_TAG_OPERATION,
+                            IPP_TAG_JOB,
                             option,
                             units,
                             xres,
@@ -470,12 +495,80 @@ gtk_cups_request_encode_option (GtkCupsRequest *request,
         }
 
       default:
-        ippAddString (request->ipp_request,
-                      IPP_TAG_OPERATION,
-                      option_tag,
-                      option,
-                      NULL,
-                      value);
+        {
+          char *values;
+          char *s;
+          int in_quotes;
+          char *next;
+          GPtrArray *strings;
+          
+          values = g_strdup (value);
+          strings = NULL;
+          in_quotes = 0;
+          
+          for (s = values, next = s; *s != '\0'; s++)
+            {
+              if (in_quotes != 2 && *s == '\'')
+                {
+                  /* skip quoted value */
+                  if (in_quotes == 0)
+                    in_quotes = 1;
+                  else
+                    in_quotes = 0;
+                }
+              else if (in_quotes != 1 && *s == '\"')
+                {
+                  /* skip quoted value */
+                  if (in_quotes == 0)
+                    in_quotes = 2;
+                  else
+                    in_quotes = 0;
+                }
+              else if (in_quotes == 0 && *s == ',')
+                {
+                  /* found delimiter, add to value array */
+                  *s = '\0';
+                  if (strings == NULL)
+                    strings = g_ptr_array_new ();
+                  g_ptr_array_add (strings, next);
+                  next = s + 1;
+                }
+              else if (in_quotes == 0 && *s == '\\' && s[1] != '\0')
+                {
+                  /* skip escaped character */
+                  s++;
+                }
+            }
+          
+          if (strings == NULL)
+            {
+              /* single value */
+              ippAddString (request->ipp_request,
+                            IPP_TAG_JOB,
+                            option_tag,
+                            option,
+                            NULL,
+                            value);
+            }
+          else
+            {
+              /* multiple values */
+              
+              /* add last value */
+              g_ptr_array_add (strings, next);
+              
+              ippAddStrings (request->ipp_request,
+                             IPP_TAG_JOB,
+                             option_tag,
+                             option,
+                             strings->len,
+                             NULL,
+                             (const char **) strings->pdata);
+              g_ptr_array_free (strings, TRUE);
+            }
+
+          g_free (values);
+        }
 
         break;
     }
@@ -534,7 +627,9 @@ _post_send (GtkCupsRequest *request)
   httpClearFields(request->http);
   httpSetField(request->http, HTTP_FIELD_CONTENT_LENGTH, length);
   httpSetField(request->http, HTTP_FIELD_CONTENT_TYPE, "application/ipp");
+#ifdef HAVE_HTTP_AUTHSTRING
   httpSetField(request->http, HTTP_FIELD_AUTHORIZATION, request->http->authstring);
+#endif
 
   if (httpPost(request->http, request->resource))
     {
@@ -543,7 +638,12 @@ _post_send (GtkCupsRequest *request)
           request->state = GTK_CUPS_POST_DONE;
           request->poll_state = GTK_CUPS_HTTP_IDLE;
 
-          gtk_cups_result_set_error (request->result, "Failed Post");
+          /* TODO: should add a status or error code for failed post */
+          gtk_cups_result_set_error (request->result,
+                                     GTK_CUPS_ERROR_GENERAL,
+                                     0,
+                                     0,
+                                     "Failed Post");
         }
 
       request->attempts++;
@@ -570,10 +670,16 @@ _post_write_request (GtkCupsRequest *request)
 
   if (ipp_status == IPP_ERROR)
     {
+      int cups_error = cupsLastError ();
       request->state = GTK_CUPS_POST_DONE;
       request->poll_state = GTK_CUPS_HTTP_IDLE;
  
-      gtk_cups_result_set_error (request->result, "%s",ippErrorString (cupsLastError ()));
+      gtk_cups_result_set_error (request->result, 
+                                 GTK_CUPS_ERROR_IPP,
+                                 ipp_status,
+                                 cups_error,
+                                 "%s", 
+                                 ippErrorString (cups_error));
       return;
     }
 
@@ -629,7 +735,12 @@ _post_write_data (GtkCupsRequest *request)
           request->state = GTK_CUPS_POST_DONE;
 	  request->poll_state = GTK_CUPS_HTTP_IDLE;
      
-          gtk_cups_result_set_error (request->result, "Error reading from cache file: %s", error->message);
+          gtk_cups_result_set_error (request->result,
+                                     GTK_CUPS_ERROR_IO,
+                                     io_status,
+                                     error->code, 
+                                     "Error reading from cache file: %s",
+                                     error->message);
 
 	  g_error_free (error);
           return;
@@ -650,10 +761,19 @@ _post_write_data (GtkCupsRequest *request)
       if (httpWrite(request->http, buffer, (int) bytes) < bytes)
 #endif /* HAVE_CUPS_API_1_2 */
         {
+          int http_errno;
+
+          http_errno = httpError (request->http);
+
           request->state = GTK_CUPS_POST_DONE;
 	  request->poll_state = GTK_CUPS_HTTP_IDLE;
      
-          gtk_cups_result_set_error (request->result, "Error writting to socket in Post %s", strerror (httpError (request->http)));
+          gtk_cups_result_set_error (request->result,
+                                     GTK_CUPS_ERROR_HTTP,
+                                     http_status,
+                                     http_errno, 
+                                     "Error writing to socket in Post %s", 
+                                     g_strerror (http_errno));
           return;
         }
     }
@@ -686,17 +806,21 @@ _post_check (GtkCupsRequest *request)
       request->state = GTK_CUPS_POST_DONE;
       request->poll_state = GTK_CUPS_HTTP_IDLE;
       
-      gtk_cups_result_set_error (request->result, "Can't prompt for authorization");
+      /* TODO: create a not implemented error code */
+      gtk_cups_result_set_error (request->result, 
+                                 GTK_CUPS_ERROR_GENERAL,
+                                 0,
+                                 0,
+                                 "Can't prompt for authorization");
       return;
     }
   else if (http_status == HTTP_ERROR)
     {
+      int error = httpError (request->http);
 #ifdef G_OS_WIN32
-      if (request->http->error != WSAENETDOWN && 
-          request->http->error != WSAENETUNREACH)
+      if (error != WSAENETDOWN && error != WSAENETUNREACH)
 #else
-      if (request->http->error != ENETDOWN && 
-          request->http->error != ENETUNREACH)
+      if (error != ENETDOWN && error != ENETUNREACH)	  
 #endif /* G_OS_WIN32 */
         {
           request->attempts++;
@@ -707,17 +831,23 @@ _post_check (GtkCupsRequest *request)
           request->state = GTK_CUPS_POST_DONE;
           request->poll_state = GTK_CUPS_HTTP_IDLE;
      
-          gtk_cups_result_set_error (request->result, "Unknown HTTP error");
+          gtk_cups_result_set_error (request->result,
+                                     GTK_CUPS_ERROR_HTTP,
+                                     http_status,
+                                     error, 
+                                     "Unknown HTTP error");
+
           return;
         }
     }
-/* TODO: detect ssl in configure.ac */
-#if HAVE_SSL
   else if (http_status == HTTP_UPGRADE_REQUIRED)
     {
       /* Flush any error message... */
       httpFlush (request->http);
 
+      cupsSetEncryption (HTTP_ENCRYPT_REQUIRED);
+      request->state = GTK_CUPS_POST_CONNECT;
+
       /* Reconnect... */
       httpReconnect (request->http);
 
@@ -727,7 +857,6 @@ _post_check (GtkCupsRequest *request)
       request->attempts++;
       goto again;
     }
-#endif 
   else if (http_status != HTTP_OK)
     {
       int http_errno;
@@ -739,7 +868,12 @@ _post_check (GtkCupsRequest *request)
       else
         {
           request->state = GTK_CUPS_POST_DONE;
-          gtk_cups_result_set_error (request->result, "HTTP Error in POST %s", strerror (http_errno));
+          gtk_cups_result_set_error (request->result,
+                                     GTK_CUPS_ERROR_HTTP,
+                                     http_status,
+                                     http_errno, 
+                                     "HTTP Error in POST %s", 
+                                     g_strerror (http_errno));
          request->poll_state = GTK_CUPS_HTTP_IDLE;
  
           httpFlush(request->http); 
@@ -788,7 +922,13 @@ _post_read_response (GtkCupsRequest *request)
 
   if (ipp_status == IPP_ERROR)
     {
-      gtk_cups_result_set_error (request->result, "%s", ippErrorString (cupsLastError()));
+      int ipp_error = cupsLastError ();
+      gtk_cups_result_set_error (request->result,  
+                                 GTK_CUPS_ERROR_IPP,
+                                 ipp_status,
+                                 ipp_error,
+                                 "%s",
+                                 ippErrorString (ipp_error));
       
       ippDelete (request->result->ipp_response);
       request->result->ipp_response = NULL;
@@ -813,7 +953,12 @@ _get_send (GtkCupsRequest *request)
 
   if (request->data_io == NULL)
     {
-      gtk_cups_result_set_error (request->result, "Get requires an open io channel");
+      gtk_cups_result_set_error (request->result,
+                                 GTK_CUPS_ERROR_IO,
+                                 G_IO_STATUS_ERROR,
+                                 G_IO_CHANNEL_ERROR_FAILED, 
+                                 "Get requires an open io channel");
+
       request->state = GTK_CUPS_GET_DONE;
       request->poll_state = GTK_CUPS_HTTP_IDLE;
 
@@ -821,7 +966,9 @@ _get_send (GtkCupsRequest *request)
     }
 
   httpClearFields(request->http);
+#ifdef HAVE_HTTP_AUTHSTRING
   httpSetField(request->http, HTTP_FIELD_AUTHORIZATION, request->http->authstring);
+#endif
 
   if (httpGet(request->http, request->resource))
     {
@@ -829,8 +976,13 @@ _get_send (GtkCupsRequest *request)
         {
           request->state = GTK_CUPS_GET_DONE;
           request->poll_state = GTK_CUPS_HTTP_IDLE;
-	  
-          gtk_cups_result_set_error (request->result, "Failed Get");
+	 
+          /* TODO: should add a status or error code for failed GET */ 
+          gtk_cups_result_set_error (request->result, 
+                                     GTK_CUPS_ERROR_GENERAL,
+                                     0,
+                                     0,
+                                     "Failed Get");
         }
 
       request->attempts++;
@@ -867,17 +1019,23 @@ _get_check (GtkCupsRequest *request)
       g_warning ("NOT IMPLEMENTED: We need to prompt for authorization in a non blocking manner");
       request->state = GTK_CUPS_GET_DONE;
       request->poll_state = GTK_CUPS_HTTP_IDLE;
- 
-      gtk_cups_result_set_error (request->result, "Can't prompt for authorization");
+
+      /* TODO: should add a status or error code for not implemented */ 
+      gtk_cups_result_set_error (request->result, 
+                                 GTK_CUPS_ERROR_GENERAL,
+                                 0,
+                                 0,
+                                 "Can't prompt for authorization");
       return;
     }
-/* TODO: detect ssl in configure.ac */
-#if HAVE_SSL
   else if (http_status == HTTP_UPGRADE_REQUIRED)
     {
       /* Flush any error message... */
       httpFlush (request->http);
 
+      cupsSetEncryption (HTTP_ENCRYPT_REQUIRED);
+      request->state = GTK_CUPS_POST_CONNECT;
+
       /* Reconnect... */
       httpReconnect (request->http);
 
@@ -887,7 +1045,6 @@ _get_check (GtkCupsRequest *request)
       request->attempts++;
       goto again;
     }
-#endif 
   else if (http_status != HTTP_OK)
     {
       int http_errno;
@@ -899,7 +1056,12 @@ _get_check (GtkCupsRequest *request)
       else
         {
           request->state = GTK_CUPS_GET_DONE;
-          gtk_cups_result_set_error (request->result, "HTTP Error in GET %s", strerror (http_errno));
+          gtk_cups_result_set_error (request->result,
+                                     GTK_CUPS_ERROR_HTTP,
+                                     http_status,
+                                     http_errno, 
+                                     "HTTP Error in GET %s", 
+                                     g_strerror (http_errno));
           request->poll_state = GTK_CUPS_HTTP_IDLE;
           httpFlush(request->http);
 
@@ -975,7 +1137,11 @@ _get_read_data (GtkCupsRequest *request)
       request->state = GTK_CUPS_POST_DONE;
       request->poll_state = GTK_CUPS_HTTP_IDLE;
     
-      gtk_cups_result_set_error (request->result, error->message);
+      gtk_cups_result_set_error (request->result,
+                                 GTK_CUPS_ERROR_IO,
+                                 io_status,
+                                 error->code, 
+                                 error->message);
       g_error_free (error);
     }
 }
@@ -998,6 +1164,24 @@ gtk_cups_result_get_response (GtkCupsResult *result)
   return result->ipp_response;
 }
 
+GtkCupsErrorType
+gtk_cups_result_get_error_type (GtkCupsResult *result)
+{
+  return result->error_type;
+}
+
+int
+gtk_cups_result_get_error_status (GtkCupsResult *result)
+{
+  return result->error_status;
+}
+
+int
+gtk_cups_result_get_error_code (GtkCupsResult *result)
+{
+  return result->error_code;
+}
+
 const char *
 gtk_cups_result_get_error_string (GtkCupsResult *result)
 {
