@@ -1,3 +1,23 @@
+/* testicontheme.c
+ * Copyright (C) 2002, 2003  Red Hat, Inc.
+ * Authors: Alexander Larsson, Owen Taylor
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ */
+
 #include <gtk/gtk.h>
 #include <stdlib.h>
 #include <string.h>
@@ -96,9 +116,9 @@ main (int argc, char *argv[])
       if (argc >= 5)
 	size = atoi (argv[4]);
       
-      icon_info = gtk_icon_theme_lookup_icon (icon_theme, argv[3], size, 0);
+      icon_info = gtk_icon_theme_lookup_icon (icon_theme, argv[3], size, GTK_ICON_LOOKUP_USE_BUILTIN);
       g_print ("icon for %s at %dx%d is %s\n", argv[3], size, size,
-	       icon_info ? gtk_icon_info_get_filename (icon_info) : "<none>");
+	       icon_info ? (gtk_icon_info_get_builtin_pixbuf (icon_info) ? "<builtin>" : gtk_icon_info_get_filename (icon_info)) : "<none>");
 
       if (icon_info) 
 	{
